package com.altinity.opsview.odw.bc;

/**
 * This file is part of Opsview
 * 
 * Opsview is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * Opsview is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Opsview; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 * 
 */

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.springframework.jdbc.datasource.DriverManagerDataSource;

import com.altinity.opsview.odw.CommandLineParams;
import com.altinity.opsview.odw.ReportConstants;
import com.altinity.opsview.odw.dao.ODWDAO;
import com.altinity.opsview.odw.dao.ODWReportConfigDAO;
import com.altinity.opsview.odw.dao.ODWReportDAO;
import com.altinity.opsview.odw.elements.ErrorsSummary;
import com.altinity.opsview.odw.exception.InvalidUsageException;
import com.altinity.opsview.odw.exception.ReportException;
import com.altinity.opsview.odw.model.ReportConfig;
import com.altinity.opsview.odw.reports.ReportCriteria;
import com.excira.utils.StringUtils;

public class BaseReportBC implements ReportConstants {
	
	protected ODWReportDAO dao;
	protected ODWReportConfigDAO configDao;
	protected Date startDate;
	protected Date endDate;
	protected Date runDate;
	protected String reportId;
	protected ODWReportLogger log;
	protected ReportCriteria reportCriteria = null;
	
	protected Document doc = null;
	
	protected String fileFormat = "jpeg";
	protected String chartsRoot;
	
	protected String outputFilePath;
	
	/**
	 * 
	 */
	public void init(CommandLineParams params) throws InvalidUsageException {
		
		// Get the start and end date for the report
		startDate = initStartDate(params);
		System.out.println("Using start date: " + startDate);
		
		endDate = initEndDate(params);
		System.out.println("Using end date: " + endDate);
		
		runDate = params.getRunDate();
		reportId = params.getReportId();
		log = new ODWReportLogger();
		
		// If username and password has been specified on 
		// command line, use for both data sources
		String userName = params.getUserName();
		String password = params.getPassword();

		if ( StringUtils.isNullOrEmptyString(userName) )
			return;
		
		// Update usernames and passwords on data sources
		// for both DAOs
		DriverManagerDataSource ds = (DriverManagerDataSource)dao.getDataSource();
		ds.setUsername(userName);
		ds.setPassword(password);
		
		DriverManagerDataSource configDs = (DriverManagerDataSource)configDao.getDataSource();
		configDs.setUsername(userName);
		configDs.setPassword(password);
	}
	
	/**
	 * 
	 */
	protected Date initStartDate(CommandLineParams params) throws InvalidUsageException {
		
		return null;
	}
	
	/**
	 * 
	 */
	protected Date initEndDate(CommandLineParams params) {
		
		return null;
	}
	
	/**
	 * 
	 */
	public Document createNewReportDocument() {
		
		doc = DocumentHelper.createDocument();
		Element report = doc.addElement("report");
		
		report.addElement("title").setText(reportCriteria.getReportName());
		report.addElement("report-type").setText(reportCriteria.getReportType());
		String selection = reportCriteria.getReportSelection();
		if ( StringUtils.notNullOrEmpty(selection) )
			report.addElement("selection").setText(selection);
		
		// Determine the report format (page size)
		// Either A4 or US
		report.addAttribute("report-format", "report" + reportCriteria.getFormat());
		report.addAttribute("report-cover-format", "cover" + reportCriteria.getFormat());
		
		SimpleDateFormat dateFormatter = new SimpleDateFormat(DATE_FORMAT);
		
		// Determine report start and end dates
		Date coverStartDate = reportCriteria.getDataStartDate();
		if ( coverStartDate == null )
			coverStartDate = reportCriteria.getReportStartDate();
		report.addElement("start-date").setText(dateFormatter.format(coverStartDate));
		
		Date coverEndDate = reportCriteria.getDataEndDate();
		if ( coverEndDate == null )
			coverEndDate = reportCriteria.getReportEndDate();
		report.addElement("end-date").setText(dateFormatter.format(coverEndDate));
		
		// Add run date
		report.addElement("run-date").setText(dateFormatter.format(Calendar.getInstance().getTime()));
		
		// Add sections
		report.addElement("report-sections");
		
		return doc;

	}	
	
	/**
	 * 
	 */
	public void hitDBs() throws ReportException {
		
		// First attempt to connect to the the config database
		hitDB(configDao);
		hitDB(dao);
	}
	
	/**
	 * 
	 */
	private void hitDB(ODWDAO daoToTest) throws ReportException {
		
		String url = ((DriverManagerDataSource)daoToTest.getDataSource()).getUrl();
		try {
			
			daoToTest.hitDB();
		}
		catch(Exception e) {
			throw new ReportException("Unable to connect to database at URL '" + url + "'. Check connection URL, username and password.");
		}
	}	
	
	/**
	 * 
	 */
	public boolean loadReporCriteria() throws ReportException {

		ReportConfig reportConfig = configDao.getReportConfigByReportId(reportId);

		reportCriteria = new ReportCriteria();
		
		// We need basic report data to be added for all 
		// cases
		// This is mainly for details displayed on the "cover"
		// Even if there is no data, we still need this basic info
		
		// Set the report start and end date
		reportCriteria.setReportStartDate(getStartDate());
		reportCriteria.setReportEndDate(getEndDate());
		
		// Set report details
		reportCriteria.setReportId(reportConfig.getId());
		reportCriteria.setReportName(reportConfig.getName());
		reportCriteria.setReportSelection(reportConfig.getSelection());
		reportCriteria.setReportType(reportConfig.getType());
		reportCriteria.setFileFormat(fileFormat);
		reportCriteria.setStartOfWeek(reportConfig.getStartOfWeek());
		reportCriteria.setReportTypeId(reportConfig.getTypeId());
		
		// Handle any problems with the report configuration
		// here - do not rethrow...just report the error and 
		// keep going
		try {
			reportCriteria.setFormat(reportConfig.getFormat());
			reportCriteria.setLabel(reportConfig.getLabel());
		}
		catch(ReportException e) {
			log.getErrors().add(e.getMessage());
		}
		
		reportCriteria.setChartsRoot(chartsRoot);
		
		// Confirm chart directory is valid
		validateDir(chartsRoot);
		
		// Now set the data start and end dates
		// If there is no data, an exception will be thrown
		// but we have already specified the basic report
		// details, allowing a basic report with errors
		// to be generated
		try {
			
			// Set the data start and end dates
			reportCriteria.setDataStartDate(dao.getDataStartDate(getStartDate(), getEndDate()));
			reportCriteria.setDataEndDate(dao.getDataEndDate(getStartDate(), getEndDate()));	
		}
		catch(ReportException e) {
			
			System.out.println(e.getMessage());
			log.getErrors().add(e.getMessage());
			
			// Return false to indicate no data found for reporting period
			return false;
		}
		
		return true;
	}	
	
	/**
	 * 
	 */
	public void setDao(ODWReportDAO dao) {
		this.dao = dao;
	}
	
	/**
	 * 
	 */
	public void setConfigDao(ODWReportConfigDAO dao) {
		this.configDao = dao;
	}

	/**
	 * 
	 */
	public ReportCriteria getReportCriteria() {
		return reportCriteria;
	}

	/**
	 * 
	 */
	public Date getEndDate() {
		return endDate;
	}

	/**
	 * 
	 */
	public Date getStartDate() {
		return startDate;
	}

	/**
	 * 
	 */
	public ODWReportLogger getLog() {
		return log;
	}

	/**
	 * 
	 */
	protected Element getReportSections() {
		return (Element) doc.selectSingleNode("/report/report-sections");
	}
	
	/**
	 * 
	 */
	public void setChartsRoot(String chartsRoot) {

		this.chartsRoot = chartsRoot;
	}	

	/**
	 * 
	 */
	public void createErrorsSummary() throws ReportException {
		
		Element reportSections = getReportSections();
		
		// Add failure section
		Element section = reportSections.addElement("section");
		section.addElement("title").setText("Errors");
		section.addAttribute("name", "errorsSummary");
		section.addAttribute("lastSection", "true");
		Element elements = section.addElement("elements");
		
		new ErrorsSummary(reportCriteria, dao, configDao, getLog()).addElement(elements);
	}
	
	/**
	 * 
	 */
	public void validateDir(String filePath) throws ReportException {
		
		File dir = new File(filePath);
		if ( !dir.exists() )
			dir.mkdir();
		
		if ( !dir.canWrite() )
			throw new ReportException("Directory " + dir.getPath() + " not writable.");
		
		if ( !dir.canRead() )
			throw new ReportException("Directory " + dir.getPath() + " not readable.");		
	}	
	
	/**
	 * 
	 */
	public void cleanUp() {
		
		// Remove all images from temp folder
		File chartsDir = new File(reportCriteria.getChartsRoot());
		if ( !chartsDir.exists() )
			return;
		
		if ( !chartsDir.isDirectory() )
			return;
		
		// Get the list of files in the chart directory
		File [] charts = chartsDir.listFiles();
		for ( File chart : charts ) {
			
			// Only delete chart images
			if ( StringUtils.substringAfter(chart.getPath(), ".").equals(fileFormat) )
				chart.delete();
		}
	}
	
	/**
	 * 
	 */
	public String getOutputFilePath() {
		
		if ( StringUtils.isNullOrEmptyString(outputFilePath) )
			throw new RuntimeException("No output file path specified");
		
		return outputFilePath;
	}
	
	/**
	 * 
	 */
	public void createReport() throws ReportException {
		
	}
}
