package com.altinity.opsview.odw.elements;

import java.text.DateFormatSymbols;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.dom4j.Element;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.CategoryLabelPositions;

import com.altinity.opsview.odw.ReportConstants;
import com.altinity.opsview.odw.bc.ODWReportLogger;
import com.altinity.opsview.odw.dao.ODWDAO;
import com.altinity.opsview.odw.dao.ODWReportConfigDAO;
import com.altinity.opsview.odw.dao.ODWReportDAO;
import com.altinity.opsview.odw.model.ServiceCheckGroup;
import com.altinity.opsview.odw.model.WeeklyServiceAvailability;
import com.altinity.opsview.odw.reports.ReportCriteria;
import com.excira.jfreechart.BarChart;
import com.excira.jfreechart.BarData;
import com.excira.utils.DateUtils;

public class WeeklyServiceDetail extends AbstractReportElement implements ReportConstants {

	protected List <ServiceCheckGroup> serviceCheckGroups;
	private Map <String, List<BarData>> allData;
	protected Map <String, List<WeeklyServiceAvailability>> availByText;
	
	/**
	 *
	 */
	public WeeklyServiceDetail(ReportCriteria rc, ODWReportDAO dao, ODWReportConfigDAO configDao, 
			ODWReportLogger log, List <ServiceCheckGroup> groups) {
		super(rc, dao, configDao, log);
		
		this.serviceCheckGroups = groups;
	}

	/**
	 * 
	 */
	public Element addElement(Element elements) {
		
		Element detailEl = elements.addElement("element");
		detailEl.addElement("title").setText("Weekly Availability Detail");
		detailEl.addAttribute("name", "weeklyDetail");
		
		// Handle case where data has been found
		if ( !allData.isEmpty() ) {
			
			// Otherwise data has been found, create charts
			Element scsEl = detailEl.addElement("servicechecks");
					
			int chartIndex = 0;
			for ( String text : allData.keySet() ) {
				
				Element scEl = scsEl.addElement("servicecheck");
				
				// Add page break if this is not the last
				// servicecheck
				if ( chartIndex < allData.size() - 1 )
					scEl.addAttribute("break-after", "page");
				
				List<BarData> data = allData.get(text);
				
				 // Pad bar data if necessary
				padDataAcrossWeek(data, availByText.get(text));				
				
				// Add chart for this servicecheck
				Element chartEl = scEl.addElement("chart");
				chartEl.addElement("title").setText(text);
				chartEl.addElement("subTitle").setText("DAILY AVAILABILITY");
				addChart(chartEl, data, "weeklyDetailChart" + chartIndex, CategoryLabelPositions.STANDARD);
				
				// Add table for this service check
				addPercentageTable(scEl, text, "DAILY BREAKDOWN");
				
				chartIndex++;
			}
			
		}
		// Otherwise no data has been found
		else {
			
			// Add empty element
			detailEl.addElement("empty").setText("true");
			
			// Report error
			log.getErrors().add("No data found for module 'Weekly Availability Detail'");
		}
		
		return detailEl;
	}
	
	/**
	 * 
	 */
	private List<BarData> padDataAcrossWeek(List<BarData> data, List<WeeklyServiceAvailability> availList) {
		
		List <String> dayNames = Arrays.asList(new DateFormatSymbols().getWeekdays());
		
		// Report variables
		int startOfWeek = rc.getStartOfWeek();
		Date reportStartDate = rc.getReportStartDate();
		Date reportEndDate = rc.getReportEndDate();
		
		// Now pad the beginning of the list
		// for any missing days
		
		// Get the first day of the week
		Calendar calStart = DateUtils.getConfiguredCalendar(4, startOfWeek, reportStartDate);
		
		// Get the first day of data in the report
		Calendar calReportFirst = DateUtils.getConfiguredCalendar(4, startOfWeek, reportStartDate);		
		int firstInReport = dayNames.indexOf((data.get(0).getColumnKey()));
		calReportFirst.set(Calendar.DAY_OF_WEEK, firstInReport);
		Date reportFirstDate = calReportFirst.getTime();

		// Now pad any missing days between the report start day
		// and the data start day
		int barIndex = 0;
		int availIndex = 0;
		while ( DateUtils.isBefore(calStart.getTime(), reportFirstDate) ) {
			
			int dowToAdd = calStart.get(Calendar.DAY_OF_WEEK);
			
			// Pad bar data
			createBlankBarData(data, barIndex, dayNames.get(dowToAdd));
			
			// Pad table data
			createBlankAvail(availList, availIndex, dowToAdd);
			
			// Increment calendar and index
			calStart.add(Calendar.DAY_OF_WEEK, 1);
			barIndex += 3;
			availIndex++;
		}		
		
		// Now pad out the end of the list
		// for any missing days
		
		// Get the last day of the week
		Calendar calEnd = DateUtils.getConfiguredCalendar(4, startOfWeek, reportEndDate);	
		Date actualEndDate = calEnd.getTime();
		
		// Get the last day in the report so far
		BarData lastBarData = data.get(data.size() - 1);
		Calendar calReportLast = DateUtils.getConfiguredCalendar(4, startOfWeek, reportStartDate);
		int lastInReport = dayNames.indexOf(lastBarData.getColumnKey());

		// Increment last day in report as the last day should
		// already be included
		// Do not increment if last day in report is also last
		// day in reporting period as this will roll the date
		// back to the report start date and overwrite data
		int lastDayOfReport = DateUtils.getLastDayOfWeek(startOfWeek);
		if ( lastInReport != lastDayOfReport)
			lastInReport++;
		calReportLast.set(Calendar.DAY_OF_WEEK, lastInReport);
		
		// If the last day in the report is not the same as the actual report
		// end day, then pad data
		// Or, if the last day in the report is the same day as the actual
		// report end day AND there is no data for the last day, then pad
		boolean sameAsLast = DateUtils.isDateEqual(calReportLast.getTime(), actualEndDate);
		boolean dataOnFinalDay = (dayNames.indexOf(lastBarData.getColumnKey()) == lastDayOfReport);
		if ( !sameAsLast || (sameAsLast && !dataOnFinalDay) ) {		
			while ( DateUtils.isBefore(calReportLast.getTime(), actualEndDate) 
					|| DateUtils.isDateEqual(calReportLast.getTime(), actualEndDate) ) {
				
				int dowToAdd = calReportLast.get(Calendar.DAY_OF_WEEK);
				
				// Pad bar data
				createBlankBarData(data, data.size(), dayNames.get(dowToAdd));
				
				// Pad table data
				createBlankAvail(availList, availList.size(), dowToAdd);
				
				// Increment calendar and index
				calReportLast.add(Calendar.DAY_OF_WEEK, 1);
			}
		}
		
		return data;
	}	
	
	/**
	 * 
	 */
	private void createBlankBarData(List<BarData> data, int index, Comparable xKey) {
		
		BarData barData = new BarData(0d, ODWReportDAO.PERCENT_OK_INDEX, xKey);
		data.add(index, barData);
		index++;
		
		barData = new BarData(0d, ODWReportDAO.PERCENT_SCHED_INDEX, xKey);
		data.add(index, barData);
		index++;
		
		barData = new BarData(0d, ODWReportDAO.PERCENT_UNSCHED_INDEX, xKey);
		data.add(index, barData);
	}	
	
	/**
	 * 
	 */
	private void createBlankAvail(List<WeeklyServiceAvailability> availList, int index, int dow) {
		
		WeeklyServiceAvailability avail = new WeeklyServiceAvailability();
		avail.setDayOfWeek(dow);
		
		availList.add(index, avail);
	}		
	
	/**
	 * 
	 */
	protected void addPercentageTable(Element parentEl, String text, String title) {
		
		Element tableEl = parentEl.addElement("availPercentage");
		tableEl.addElement("title").setText(title);
		
		Element appEl = tableEl.addElement("cols");
		Element okEl = tableEl.addElement("oks");
		Element schedEl = tableEl.addElement("scheds");
		Element unschedEl = tableEl.addElement("unscheds");
		
		DecimalFormat percentFormat = new DecimalFormat(PERCENT_FORMAT);
		String[] dayNames = new DateFormatSymbols().getWeekdays();
		
		List <WeeklyServiceAvailability> weekAvail = availByText.get(text);
		for ( WeeklyServiceAvailability avail : weekAvail ) {
		
			appEl.addElement("col").setText(dayNames[avail.getDayOfWeek()]);
			
			// If there is no availability for this day, just add empty text
			if ( avail.isEmpty() ) {
			
				okEl.addElement("okValue").setText("");
				schedEl.addElement("schedValue").setText("");
				unschedEl.addElement("unschedValue").setText("");
				
				continue;
			}
			
			// Otherwise add percentages
			okEl.addElement("okValue").setText(percentFormat.format(avail.getPercentOk()));
			schedEl.addElement("schedValue").setText(percentFormat.format(avail.getPercentScheduled()));
			unschedEl.addElement("unschedValue").setText(percentFormat.format(avail.getPercentUnscheduled()));
		}
	}	

	/**
	 * 
	 */
	public void runQuery() {
		
		String[] dayNames = new DateFormatSymbols().getWeekdays();
		
		allData = new LinkedHashMap<String, List<BarData>>();
		availByText = new LinkedHashMap<String, List<WeeklyServiceAvailability>>();
		
		// Now query availability for each servicecheck
		for ( ServiceCheckGroup group : serviceCheckGroups ) {
			
			// Confirm the group has a most recent
			String text = group.getText();
			
			// Get the list of daily availability for this servicecheck
			List<WeeklyServiceAvailability> summaries = dao.getDailyAvailabilityForServiceCheck(rc, group.getIds());
			if ( summaries.isEmpty() )
				continue;

			// Generate bar data for each day returned
			List <BarData> data = new ArrayList<BarData>();
			for ( WeeklyServiceAvailability summary : summaries ) {
				
				String dow = dayNames[summary.getDayOfWeek()];
				
				BarData barData = new BarData(summary.getPercentOk(), ODWDAO.PERCENT_OK_INDEX, dow);
				data.add(barData);
				
				barData = new BarData(summary.getPercentScheduled(), ODWDAO.PERCENT_SCHED_INDEX, dow);
				data.add(barData);
				
				barData = new BarData(summary.getPercentUnscheduled(), ODWDAO.PERCENT_UNSCHED_INDEX, dow);
				data.add(barData);
				
				List <WeeklyServiceAvailability> weekSummaries = availByText.get(text);
				if ( weekSummaries == null ) {
					
					weekSummaries = new ArrayList<WeeklyServiceAvailability>();
					availByText.put(text, weekSummaries);
				}
				weekSummaries.add(summary);
			}
			
			allData.put(text, data);
		}
	}
	
	
	/**
	 * 
	 */
	protected void addChart(Element parentEl, List<BarData> data, String fileNameBase, CategoryLabelPositions position) {
		
		BarChart barChart = new BarChart();
		barChart.createDomainAxis("", position);
		barChart.createRangeAxis(data, "", 25d);
		barChart.setHideLegend(true);
		JFreeChart chart = barChart.createChart("", "", data, false);
		String fileName = saveImageToTempDir(chart, fileNameBase);
		
		parentEl.addElement("filePath").setText(fileName);
	}	
}
